/*
 * View model for OctoPrint-Ttprintercalibration
 *
 * Author: Francois Guyot
 * License: AGPLv3
 */
$(function () {
    function TtprintercalibrationViewModel(parameters) {
        var self = this;

        self.settingsViewModel = parameters[0];

        self.bed_size_x = ko.observable();
        self.bed_size_y = ko.observable();
        self.bed_size_z = ko.observable();
        self.origin_check = ko.observable();
        self.filament_used = ko.observable();
        self.printer_name = ko.observable();
        self.relative_positioning = ko.observable();
        self.abl_method_retra = ko.observable();
        self.abl_method_temp = ko.observable();
        self.nozzle_size = ko.observable();
        self.filament_diameter = ko.observable();
        self.jerk_x = ko.observable();
        self.jerk_y = ko.observable();
        self.heating_position_x = ko.observable();
        self.heating_position_y = ko.observable();
        self.heating_position_z = ko.observable();

        let nozzle_size;
        let baseHeight;
        let stageHeightRetra;
        let spanValRetra;

        self.variable = {};

        let modifiedIds = [];
        let modifiableIds = ["printerName", "filamentUsed", "filamentDiameter", "bedSizeX", "bedSizeY", "bedSizeZ", "nozzleSize", "relativePositioning", "originCheck", "heatingPositionX", "heatingPositionY", "heatingPositionZ"];
        let modifiedElements = [];
        let modifiableElements = ["printer_name", "filament_used", "filament_diameter", "bed_size_x", "bed_size_y", "bed_size_z", "nozzle_size", "relative_positioning", "origin_check", "heating_position_x", "heating_position_y", "heating_position_z"];

        let restrictedIdClassic = ["#bedSizeX", "#bedSizeY", "#bedSizeZ", "#nozzlePidTemp", "#bedPidTemp", "#knobBorderX", "#knobBorderY", "heatingPositionX", "heatingPositionY", "heatingPositionZ"];
        self.allowedArrayClassic = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'];
        self.appendedArray = [];
        let restrictedIdComma = ["#filamentDiameter"];
        self.allowedArrayComma = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.'];
        self.appendedArrayComma = [];

        let restrictedInputs = ["#printerName", "#filamentUsed", "#filamentDiameter", "#bedSizeX", "#bedSizeY",
            "#bedSizeZ", "#nozzleSize", "#heatingPositionX", "#heatingPositionY", "#heatingPositionZ"];
        let saveInputs = ["printer_name", "filament_used", "filament_diameter", "bed_size_x", "bed_size_y",
            "bed_size_z", "nozzle_size", "heating_position_x", "heating_position_y", "heating_position_z"];

        let restrictedCheckbox = ["#relativePositioning", "#originCheck"];
        let saveCheckbox = ["relative_positioning", "origin_check"];

        self.onBeforeBinding = function() {
            self.bed_size_x(self.settingsViewModel.settings.plugins.TTPrinterCalibration.bed_size_x());
            self.bed_size_y(self.settingsViewModel.settings.plugins.TTPrinterCalibration.bed_size_y());
            self.bed_size_z(self.settingsViewModel.settings.plugins.TTPrinterCalibration.bed_size_z());
            self.origin_check(self.settingsViewModel.settings.plugins.TTPrinterCalibration.origin_check());
            self.filament_used(self.settingsViewModel.settings.plugins.TTPrinterCalibration.filament_used());
            self.printer_name(self.settingsViewModel.settings.plugins.TTPrinterCalibration.printer_name());
            self.relative_positioning(self.settingsViewModel.settings.plugins.TTPrinterCalibration.relative_positioning());
            self.nozzle_size(self.settingsViewModel.settings.plugins.TTPrinterCalibration.nozzle_size());
            self.filament_diameter(self.settingsViewModel.settings.plugins.TTPrinterCalibration.filament_diameter());
            self.jerk_x(self.settingsViewModel.settings.plugins.TTPrinterCalibration.jerk_x());
            self.jerk_y(self.settingsViewModel.settings.plugins.TTPrinterCalibration.jerk_y());
            self.heating_position_x(self.settingsViewModel.settings.plugins.TTPrinterCalibration.heating_position_x());
            self.heating_position_y(self.settingsViewModel.settings.plugins.TTPrinterCalibration.heating_position_y());
            self.heating_position_z(self.settingsViewModel.settings.plugins.TTPrinterCalibration.heating_position_z());

            document.getElementById("plusz").disabled = true
            document.getElementById("minusz").disabled = true
            document.getElementById("step2").disabled = true
            document.getElementById("step3").disabled = true
            document.getElementById("step4").disabled = true
        }

        self.onAfterBinding = function() {
            $(restrictedInputs.join(",")).each(function () {
                saveSettingsSetup(this);
            });
            $(restrictedCheckbox.join(",")).each(function() {
                saveSettingsSetupCheckbox(this);
            });
            self.flow = 1;
            self.resetProcedure();
        }

        /*self.onEventSettingsUpdated = function() {
            let ar = ["nozzle_pid_temp", "bed_size_x", "bed_size_y"]; //These parameters could mess things if not set correctly
            if (self.nozzle_pid_temp() >= 170) {
                ar.splice(ar.indexOf("nozzle_pid_temp"), 1);
            }
            if (self.bed_size_x() >= 100) {
                ar.splice(ar.indexOf("bed_size_x"), 1);
            }
            if (self.bed_size_y() >= 100) {
                ar.splice(ar.indexOf("bed_size_y"), 1);
            }
            if (ar.length !== 0) {
                self.notify = new PNotify({
                    title: 'TTPrinterCalibration',
                    text: 'Be careful! these parameters may be off! ' + ar,
                    hide: true
                });
            }
        }*/

        $(restrictedInputs.join(",")).on("input", function() {
            saveSettingsSetup(this);
        });

        function saveSettingsSetup(element) {
            let saveSettings = saveInputs[restrictedInputs.indexOf('#' + element.id)]
            self.variable[saveSettings] = element.value
            OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {
                [saveSettings]: element.value})
        }

        $(restrictedCheckbox.join(",")).on("click", function() {
            saveSettingsSetupCheckbox(this);
        });

        function saveSettingsSetupCheckbox(element) {
            let saveSettingsCheckbox = saveCheckbox[restrictedCheckbox.indexOf('#' + element.id)];
            OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {
                [saveSettingsCheckbox]: element.checked})
            self.variable[saveSettingsCheckbox] = element.checked;
            if (saveSettingsCheckbox === "origin_check") {
                if (element.checked) {
                    document.getElementById("bedSizeY").disabled = true;
                    self.bed_center_x = 0;
                    self.bed_center_y = 0;
                } else {
                    document.getElementById("bedSizeY").disabled = false;
                    self.bed_center_x = Math.round(self.variable.bed_size_x / 2);
                    self.bed_center_y = Math.round(self.variable.bed_size_y / 2);
                }
            }
        }

        $("#bedSizeX").on("input", function () {
            if (self.variable.origin_check) {
                document.getElementById("bedSizeY").value = this.value;
            }
        });

        self.save_changes = function() {
            /*if (document.getElementById("bedSizeZ").style.backgroundColor === "yellow" && self.bed_size_z()>=1) {
                if (spanVal*self.nozzle_size()/2>=self.bed_size_z()) {
                    self.notify = new PNotify({
                        title: 'TTPrinterCalibration',
                        text: 'The temperature tower height was lowered according to your new Printer Height Z',
                        type: 'alert',
                        hide: true,
                        buttons: {
                            closer: true,
                            sticker: false
                        },
                    });
                }
                while (spanVal*self.nozzle_size()/2>=self.bed_size_z()) {
                    self.removeTemp();
                    self.removeRetraction();
                }
            }*/
        }

        let readyState;
        OctoPrint.printer.getFullState().done(function(response) {
            readyState = response.sd.ready;
            return readyState;
        });

        self.whitespace = document.createElement("span");
        self.whitespace.className = "whitespace"
        self.whitespace2 = document.createElement("span");
        self.whitespace2.className = "whitespace2"

        $(restrictedIdClassic.join(",")).keypress(function(e) {
            let readId = e.target.id;
            let readKey = e.originalEvent.key;
            if (restrictedIdClassic.includes("#" + readId) && self.allowedArrayClassic.includes(readKey)) {
            } else {
                return false;
            }
        });
        $(restrictedIdComma.join(",")).keypress(function(e) {
            let readId = e.target.id;
            let readKey = e.originalEvent.key;
            if (restrictedIdComma.includes("#" + readId) && self.allowedArrayComma.includes(readKey)) {
            } else {
                return false;
            }
        });

        self.stageAvailable = function() {
            $(self.appendedArray.join(",")).keypress(function (e) {
                let readId = e.target.id;
                let readKey = e.originalEvent.key;
                if (self.appendedArray.includes("#" + readId) && self.allowedArrayClassic.includes(readKey)) {
                } else {
                    return false;
                }
            });
            $(self.appendedArrayComma.join(",")).keypress(function (e) {
                let readId = e.target.id;
                let readKey = e.originalEvent.key;
                if (self.appendedArrayComma.includes("#" + readId) && self.allowedArrayComma.includes(readKey)) {
                } else {
                    return false;
                }
            });
            setTimeout(self.stageAvailable, 0);
        }

        self.getReturningPosition = function(array) {
            let valueReturn = 0;
            for (let i=0; i<array.length; i++) {
                if (array[i] !== 'null') {
                    valueReturn = valueReturn + parseFloat(array[i]);
                }
            }
            return -valueReturn.toFixed(3);
        }

        self.getFullFilename = function(filename) {
            return self.variable.printer_name + "_" + self.variable.filament_used + "_" + self.variable.nozzle_size + "_" + filename;
        }

        function noStageMessage() {
            self.notify = new PNotify({
                title: 'TTPrinterCalibration',
                text: 'Please add at least one stage first!',
                type: 'alert',
                hide: true,
                buttons: {
                    closer: true,
                    sticker: false
                },
            });
        }

        /*document.getElementById("originCheck").onclick = function() {
            if (document.getElementById("originCheck").checked) {
                document.getElementById("bedSizeY").disabled = true;
                document.getElementById("bedSizeY").value = self.bed_size_x();
            }
            else {
                document.getElementById("bedSizeY").disabled = false;
            }
            document.getElementById("originCheck").style.outline = "10px solid yellow";
            save_warning_text();
        }*/

        let extruded_length;
        let last_feed_rate_calculation = 0;
        let feed_rate = 0;
        let i, j;
        let text2;
        let first_part = [];
        first_part[1] = []; first_part[2] = []; first_part[3] = []; first_part[4] = []; first_part[5] = [];
        let new_x_length, new_y_length, new_z_length;
        let offset_x_value, offset_y_value;
        let second_part_x_coord, second_part_y_coord;
        let mid_square_x, mid_square_y;
        let start_gcode;

        let procedureStarted = false

        document.getElementById("step1").onclick = function() {
            procedureStarted = true
            OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {'procedureStarted' : true})
            //OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {'current_z_offset' : "undefined"})
        }

        /*self.onServerDisconnect = function() {
            if (procedureStarted) {
                OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {'procedureStarted' : true})
            } else {
                OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {'procedureStarted' : false})
            }
        }*/

        self.resetProcedure = function() {
            //OctoPrint.control.sendGcode("M851")
            document.getElementById("step1").disabled = false
            document.getElementById("step2").disabled = true
            document.getElementById("step3").disabled = true
            document.getElementById("step4").disabled = true
            document.getElementById("plusz").disabled = true
            document.getElementById("minusz").disabled = true
            OctoPrint.settings.getPluginSettings('TTPrinterCalibration').done(function (response) {
                if (response["procedureStarted"] === true) {
                    if (typeof response["current_z_offset"] !== "undefined") {
                        OctoPrint.control.sendGcode(["M851 Z" + response["current_z_offset"], "M500"])
                        self.notify = new PNotify({
                            title: 'TTPrinterCalibration',
                            text: 'The procedure was reset and your z offset value was restored to Z' + response["current_z_offset"],
                            type: 'info',
                            hide: true,
                            buttons: {
                                closer: true,
                                sticker: false
                            },
                        });
                    } else {
                        self.notify = new PNotify({
                            title: 'TTPrinterCalibration',
                            text: 'The procedure was reset',
                            type: 'info',
                            hide: true,
                            buttons: {
                                closer: true,
                                sticker: false
                            },
                        });
                    }
                    OctoPrint.settings.savePluginSettings('TTPrinterCalibration', {'procedureStarted' : false})
                }
            })
        }

        function get_diagonal_pos(nozzle_size) {
            if (nozzle_size === 0.4) {
                offset_x_value = 0.706;
                offset_y_value = -0.161;
                second_part_x_coord = [offset_x_value, '-0.546999999999997', '0', '1.113', '0.564999999999998', '-1.678', '0', '2.244', '0.566000000000003', '-2.81', '0', '3.375', '0.566000000000003', '-3.941', '0', '4.50700000000001', '0.564999999999998', '-5.072', '0', '5.63800000000001', '0.565999999999988', '-6.20399999999999', '0', '6.76900000000001', '0.565999999999988', '-7.33499999999999', '0', '7.901', '0.566000000000003', '-8.467', '0', '9.032', '0.566000000000003', '-9.598', '0', '10.164', '0.564999999999998', '-10.729', '0', '11.295', '0.566000000000003', '-11.861', '0', '12.426', '0.566000000000003', '-12.992', '0', '13.558', '0.565000000000012', '-14.123', '0', '14.689', '0.566000000000003', '-15.255', '0', '15.82', '0.566000000000003', '-16.386', '0', '16.952', '0.564999999999998', '-17.517', '0', '18.083', '0.566000000000003', '-18.649', '0', '19.215', '0.564999999999998', '-19.78', '0', '20.346', '0.566000000000003', '-20.912', '0', '21.477', '0.565999999999988', '-22.043', '0', '22.609', '0.564999999999998', '-23.174', '0', '23.74', '0.566000000000003', '-24.306', '0', '24.871', '0.566000000000003', '-25.437', '0', '26.003', '0.565000000000026', '-26.568', '0', '27.134', '0.566000000000003', '-27.7', '0.566999999999993', '27.133', '0', '-26.568', '0.565999999999988', '26.002', '0', '-25.436', '0.564999999999998', '24.871', '0', '-24.305', '0.566000000000003', '23.739', '0', '-23.173', '0.564999999999998', '22.608', '0', '-22.042', '0.566000000000003', '21.476', '0', '-20.911', '0.566000000000003', '20.345', '0', '-19.779', '0.565000000000012', '19.214', '0', '-18.648', '0.566000000000003', '18.082', '0', '-17.517', '0.566000000000003', '16.951', '0', '-16.385', '0.564999999999998', '15.82', '0', '-15.254', '0.566000000000003', '14.688', '0', '-14.123', '0.566000000000003', '13.557', '0', '-12.991', '0.566000000000003', '12.425', '0', '-11.86', '0.566000000000003', '11.294', '0', '-10.728', '0.565000000000012', '10.163', '0', '-9.59699999999999', '0.566000000000003', '9.03099999999999', '0', '-8.46599999999999', '0.566000000000003', '7.89999999999999', '0', '-7.33399999999999', '0.564999999999998', '6.76899999999999', '0', '-6.20299999999999', '0.566000000000003', '5.63699999999999', '0', '-5.07199999999999', '0.566000000000003', '4.50599999999999', '0', '-3.93999999999998', '0.565999999999988', '3.374', '0', '-2.809', '0.566000000000003', '2.24299999999999', '0', '-1.67699999999999', '0.564999999999998', '1.11199999999999', '0', '-0.545999999999992', '0'];
                second_part_y_coord = [offset_y_value, '-0.546999999999997', '-0.564999999999998', '1.11199999999999', '0', '-1.678', '-0.566000000000003', '2.244', '0', '-2.81', '-0.564999999999984', '3.37499999999999', '0', '-3.94099999999999', '-0.566000000000003', '4.50699999999999', '0', '-5.07199999999999', '-0.566000000000003', '5.63799999999999', '0', '-6.20399999999999', '-0.564999999999998', '6.76899999999999', '0', '-7.33499999999999', '-0.566000000000003', '7.901', '0', '-8.46599999999999', '-0.566000000000003', '9.032', '0', '-9.59799999999998', '-0.564999999999998', '10.163', '0', '-10.729', '-0.566000000000003', '11.295', '0', '-11.86', '-0.566000000000003', '12.426', '0', '-12.992', '-0.566000000000003', '13.558', '0', '-14.123', '-0.566000000000003', '14.689', '0', '-15.255', '-0.564999999999998', '15.82', '0', '-16.386', '-0.565999999999988', '16.952', '0', '-17.517', '-0.565999999999988', '18.083', '0', '-18.649', '-0.564999999999998', '19.214', '0', '-19.78', '-0.566000000000003', '20.346', '0', '-20.911', '-0.566000000000003', '21.477', '0', '-22.043', '-0.564999999999998', '22.608', '0', '-23.174', '-0.566000000000003', '23.74', '0', '-24.306', '-0.565000000000012', '24.871', '0', '-25.437', '-0.566000000000003', '26.003', '0', '-26.568', '-0.566000000000003', '27.134', '0', '-27.7', '0.00100000000000477', '27.133', '-0.565999999999974', '-26.567', '0', '26.002', '-0.565999999999974', '-25.436', '0', '24.87', '-0.565000000000012', '-24.305', '0', '23.739', '-0.566000000000003', '-23.173', '0', '22.608', '-0.566000000000003', '-22.042', '0', '21.476', '-0.564999999999998', '-20.911', '0', '20.345', '-0.566000000000003', '-19.779', '0', '19.214', '-0.566000000000003', '-18.648', '0', '18.082', '-0.566000000000003', '-17.516', '0', '16.951', '-0.566000000000003', '-16.385', '0', '15.819', '-0.564999999999998', '-15.254', '0', '14.688', '-0.565999999999988', '-14.122', '0', '13.557', '-0.565999999999988', '-12.991', '0', '12.425', '-0.564999999999998', '-11.86', '0', '11.294', '-0.566000000000003', '-10.728', '0', '10.163', '-0.566000000000003', '-9.59699999999999', '0', '9.03099999999999', '-0.564999999999998', '-8.46599999999999', '0', '7.89999999999999', '-0.566000000000003', '-7.33399999999999', '0', '6.768', '-0.565000000000012', '-6.20299999999999', '0', '5.637', '-0.566000000000003', '-5.071', '0', '4.506', '-0.566000000000003', '-3.94', '0', '3.374', '-0.564999999999998', '-2.809', '0', '2.24299999999999', '-0.566000000000003', '-1.67699999999999', '0', '1.11199999999999', '-0.566000000000003', '-0.545999999999992', '-0,35'];
            }
            if (nozzle_size === 0.6) {
                offset_x_value = 0.475;
                offset_y_value = -0.241;
                second_part_x_coord = [offset_x_value, '-0.236000000000004', '0', '1.084', '0.849000000000004', '-1.93300000000001', '0', '2.78100000000001', '0.84899999999999', '-3.63', '0', '4.47799999999999', '0.849000000000004', '-5.327', '0', '6.175', '0.849000000000004', '-7.024', '0', '7.872', '0.849000000000004', '-8.721', '0', '9.569', '0.849000000000004', '-10.418', '0', '11.266', '0.84899999999999', '-12.115', '0', '12.964', '0.847999999999999', '-13.812', '0', '14.661', '0.847999999999999', '-15.509', '0', '16.358', '0.847999999999999', '-17.206', '0', '18.055', '0.847999999999999', '-18.903', '0', '19.752', '0.847999999999999', '-20.6', '0', '21.449', '0.847999999999999', '-22.297', '0', '23.146', '0.848000000000013', '-23.994', '0', '24.843', '0.847999999999985', '-25.691', '0', '26.54', '0', '-25.691', '0.849000000000004', '24.842', '0', '-23.994', '0.84899999999999', '23.145', '0', '-22.296', '0.847999999999999', '21.448', '0', '-20.599', '0.847999999999999', '19.751', '0', '-18.902', '0.847999999999999', '18.054', '0', '-17.205', '0.848000000000013', '16.357', '0', '-15.508', '0.847999999999999', '14.66', '0', '-13.811', '0.847999999999999', '12.963', '0', '-12.114', '0.847999999999999', '11.266', '0', '-10.417', '0.847999999999999', '9.569', '0', '-8.72', '0.847999999999999', '7.872', '0', '-7.02300000000001', '0.849000000000004', '6.17400000000001', '0', '-5.32600000000001', '0.849000000000004', '4.477', '0', '-3.629', '0.849000000000004', '2.78', '0', '-1.93200000000002', '0.849000000000018', '1.083', '0', '-0.235000000000014', '0'];
                second_part_y_coord = [offset_y_value, '-0.235000000000014', '-0.84899999999999', '1.084', '0', '-1.93299999999999', '-0.848000000000013', '2.78100000000001', '0', '-3.63000000000001', '-0.847999999999999', '4.47800000000001', '0', '-5.32700000000001', '-0.847999999999999', '6.17500000000001', '0', '-7.024', '-0.847999999999999', '7.872', '0', '-8.721', '-0.847999999999999', '9.569', '0', '-10.418', '-0.847999999999999', '11.266', '0', '-12.115', '-0.847999999999999', '12.963', '0', '-13.812', '-0.847999999999999', '14.66', '0', '-15.509', '-0.849000000000004', '16.358', '0', '-17.206', '-0.849000000000004', '18.055', '0', '-18.903', '-0.849000000000004', '19.752', '0', '-20.6', '-0.849000000000004', '21.449', '0', '-22.297', '-0.84899999999999', '23.146', '0', '-23.994', '-0.849000000000004', '24.843', '0', '-25.691', '-0.849000000000004', '26.54', '-0.849000000000018', '-25.69', '0', '24.842', '-0.84899999999999', '-23.993', '0', '23.145', '-0.849000000000004', '-22.296', '0', '21.448', '-0.849000000000004', '-20.599', '0', '19.751', '-0.84899999999999', '-18.902', '0', '18.054', '-0.849000000000004', '-17.205', '0', '16.357', '-0.849000000000004', '-15.508', '0', '14.66', '-0.849000000000004', '-13.811', '0', '12.963', '-0.849000000000004', '-12.114', '0', '11.265', '-0.847999999999999', '-10.417', '0', '9.568', '-0.847999999999999', '-8.72', '0', '7.871', '-0.847999999999999', '-7.023', '0', '6.17399999999999', '-0.847999999999999', '-5.32599999999999', '0', '4.47699999999999', '-0.847999999999999', '-3.62899999999999', '0', '2.78', '-0.847999999999999', '-1.932', '0', '1.083', '-0.847999999999999', '-0.234999999999999', '-0.530000000000001'];
            }
            if (nozzle_size === 0.8) {
                offset_x_value = 0.809;
                offset_y_value = -0.321;
                second_part_x_coord = [offset_x_value, '-0.490000000000009', '0', '1.62100000000001', '1.13199999999999', '-2.753', '0', '3.884', '1.131', '-5.015', '0', '6.14700000000001', '1.131', '-7.27800000000001', '0', '8.41', '1.131', '-9.541', '0', '10.672', '1.13200000000001', '-11.804', '0', '12.935', '1.131', '-14.066', '0', '15.198', '1.131', '-16.329', '0', '17.461', '1.131', '-18.592', '0', '19.723', '1.13200000000001', '-20.855', '0', '21.986', '1.131', '-23.117', '0', '24.249', '1.131', '-25.38', '1.13200000000001', '24.248', '0', '-23.117', '1.13199999999999', '21.985', '0', '-20.854', '1.13200000000001', '19.722', '0', '-18.591', '1.131', '17.46', '0', '-16.328', '1.131', '15.197', '0', '-14.066', '1.13199999999999', '12.934', '0', '-11.803', '1.13200000000001', '10.671', '0', '-9.53999999999999', '1.131', '8.40899999999999', '0', '-7.27699999999999', '1.131', '6.14599999999999', '0', '-5.01499999999999', '1.13199999999999', '3.883', '0', '-2.752', '1.13100000000001', '1.62099999999998', '0', '-0.489000000000004'];
                second_part_y_coord = [offset_y_value, '-0.489999999999981', '-1.131', '1.62099999999998', '0', '-2.75299999999999', '-1.131', '3.88399999999999', '0', '-5.01499999999999', '-1.13200000000001', '6.14699999999999', '0', '-7.27799999999999', '-1.131', '8.40899999999999', '0', '-9.541', '-1.131', '10.672', '0', '-11.804', '-1.131', '12.935', '0', '-14.066', '-1.13200000000001', '15.198', '0', '-16.329', '-1.131', '17.46', '0', '-18.592', '-1.131', '19.723', '0', '-20.855', '-1.131', '21.986', '0', '-23.117', '-1.13200000000001', '24.249', '0', '-25.38', '0.00100000000000477', '24.248', '-1.13199999999999', '-23.116', '0', '21.985', '-1.131', '-20.854', '0', '19.722', '-1.131', '-18.591', '0', '17.46', '-1.13199999999999', '-16.328', '0', '15.197', '-1.13200000000001', '-14.065', '0', '12.934', '-1.131', '-11.803', '0', '10.671', '-1.131', '-9.53999999999999', '0', '8.40899999999999', '-1.13199999999999', '-7.277', '0', '6.146', '-1.13200000000001', '-5.014', '0', '3.883', '-1.131', '-2.752', '0', '1.62', '-1.131', '-0.489000000000004'];
            }
            return [second_part_x_coord, second_part_y_coord];
        }

        /*function extruded_length_calculation(number_part, old_number_part) {
            if (number_part.includes("X") && number_part.includes("Y") && number_part.includes("Z")) {
                new_x_length = (number_part.substring(number_part.lastIndexOf("X") + 1, number_part.lastIndexOf(" "))) - (old_number_part.substring(old_number_part.lastIndexOf("X") + 1, old_number_part.lastIndexOf(" ")));
                new_y_length = (number_part.substring(number_part.lastIndexOf("Y") + 1, number_part.lastIndexOf(" "))) - (old_number_part.substring(old_number_part.lastIndexOf("Y") + 1, old_number_part.lastIndexOf(" ")));
                new_z_length = (number_part.substring(number_part.lastIndexOf("Z") + 1, number_part.length)) - (old_number_part.substring(old_number_part.lastIndexOf("Z") + 1, old_number_part.length));
            }
            else if (number_part.includes("X") && number_part.includes("Y") && !number_part.includes("Z")) {
                new_x_length = (number_part.substring(number_part.lastIndexOf("X") + 1, number_part.lastIndexOf(" "))) - (old_number_part.substring(old_number_part.lastIndexOf("X") + 1, old_number_part.lastIndexOf(" ")));
                new_y_length = (number_part.substring(number_part.lastIndexOf("Y") + 1, old_number_part.length)) - (old_number_part.substring(old_number_part.lastIndexOf("Y") + 1, old_number_part.length));
                new_z_length = 0;
            }
            else if (number_part.includes("X") && !number_part.includes("Y") && !number_part.includes("Z")) {
                new_x_length = (number_part.substring(number_part.lastIndexOf("X") + 1, old_number_part.length)) - (old_number_part.substring(old_number_part.lastIndexOf("X") + 1, old_number_part.length));
                new_y_length = 0;
                new_z_length = 0;
            }
            else if (!number_part.includes("X") && number_part.includes("Y") && !number_part.includes("Z")) {
                new_x_length = 0;
                new_y_length = (number_part.substring(number_part.lastIndexOf("Y") + 1, old_number_part.length)) - (old_number_part.substring(old_number_part.lastIndexOf("Y") + 1, old_number_part.length));
                new_z_length = 0;
            }
            else if (!number_part.includes("X") && !number_part.includes("Y") && number_part.includes("Z")) {
                new_x_length = 0;
                new_y_length = 0;
                new_z_length = (number_part.substring(number_part.lastIndexOf("Z") + 1, old_number_part.length)) - (old_number_part.substring(old_number_part.lastIndexOf("Z") + 1, old_number_part.length));
            }
                extruded_length = Math.pow(Math.pow(new_x_length,2) + Math.pow(new_y_length,2),1/2)
                feed_rate_calculation(extruded_length)
                return extruded_length
            }*/

        function feed_rate_calculation(extruded_length) {
            last_feed_rate_calculation = ((nozzle_size/2)*nozzle_size*extruded_length)/((Math.PI/4)*Math.pow(self.filament_diameter(), 2));
            feed_rate = last_feed_rate_calculation + feed_rate
            if (extruded_length === "reset") {
                last_feed_rate_calculation = 0;
                feed_rate = 0;
                return
            }
            return feed_rate
        }

        self.extruded_length_calculation_relative = function(xMovement, yMovement) {
            extruded_length = Math.pow(Math.pow(xMovement,2) + Math.pow(yMovement,2),1/2)
            if (self.flowCube) {
                feed_rate_calculation_flow_cube(extruded_length)
            } else {
                feed_rate_calculation_diagonal(extruded_length)
            }
        }

        function extruded_length_calculation_relative_withz(xMovement, yMovement, zMovement) {
            extruded_length = Math.pow(Math.pow(xMovement,2) + Math.pow(yMovement,2) + Math.pow(zMovement,2),1/2)
            feed_rate_calculation_diagonal(extruded_length)
        }

        let feed_rate_relative = 0;

        function feed_rate_calculation_diagonal(extruded_length) {
            let layer_height = self.variable.nozzle_size / 2;
            let layer_width = self.variable.nozzle_size;
            if (self.variable.relative_positioning) {
                feed_rate_relative = Math.round(self.flow * 100000 * (layer_height * layer_width * extruded_length) / ((Math.PI / 4) * Math.pow(self.variable.filament_diameter, 2))) / 100000;
            } else {
                feed_rate_relative = Math.round(self.flow * 100000 * (layer_height * layer_width * extruded_length) / ((Math.PI / 4) * Math.pow(self.variable.filament_diameter, 2))) / 100000 + self.last_feed_rate;
                self.last_feed_rate = feed_rate_relative
            }
            self.feed_rate = feed_rate_relative.toFixed(5)
            return self.feed_rate
        }
        function feed_rate_calculation_flow_cube(extruded_length) {
            let layer_height = self.variable.nozzle_size / 2;
            let layer_width = self.variable.nozzle_size;
            if (self.variable.relative_positioning) {
                feed_rate_relative = Math.round(self.flow * 100000 * (((layer_width - layer_height) * layer_height + (Math.PI * Math.pow(layer_height / 2, 2))) * extruded_length) / ((Math.PI / 4) * Math.pow(self.variable.filament_diameter, 2))) / 100000;
            } else {
                feed_rate_relative = Math.round(self.flow * 100000 * (((layer_width - layer_height) * layer_height + (Math.PI * Math.pow(layer_height / 2, 2))) * extruded_length) / ((Math.PI / 4) * Math.pow(self.variable.filament_diameter, 2))) / 100000 + self.last_feed_rate;
                self.last_feed_rate = feed_rate_relative
            }
            self.feed_rate = feed_rate_relative.toFixed(5)
            return self.feed_rate
        }

        let z=0;
        let xAbsolute = [];
        let yAbsolute = [];
        let zAbsolute = [];
        let xLastAbsolute;
        let yLastAbsolute;
        let zLastAbsolute;

        self.getAbsoluteCoordinate = function(xRelative, yRelative, zRelative, boolean) {
            if (boolean) {
                z=0;
                xAbsolute = []
                yAbsolute = [];
                zAbsolute = [];
                xLastAbsolute = xAbsolute[0] = self.first_x_pos;
                yLastAbsolute = yAbsolute[0] = self.first_y_pos;
                zLastAbsolute = zAbsolute[0] = parseFloat(self.first_z_pos) + parseFloat(self.variable.nozzle_size)/2;
            }
                if (xRelative === "null") {
                    xAbsolute[z + 1] = "null";
                } else {
                    xAbsolute[z + 1] = (parseFloat(xLastAbsolute)+parseFloat(xRelative)).toFixed(3)
                    xLastAbsolute = xAbsolute[z + 1]
                }
                if (yRelative === "null") {
                    yAbsolute[z + 1] = "null";
                } else {
                    yAbsolute[z + 1] = (parseFloat(yLastAbsolute)+parseFloat(yRelative)).toFixed(3)
                    yLastAbsolute = yAbsolute[z + 1]
                }
                if (zRelative === "null") {
                    zAbsolute[z + 1] = "null";
                } else {
                    if (zRelative !== undefined) {
                        zAbsolute[z + 1] = (parseFloat(zLastAbsolute) + parseFloat(zRelative)).toFixed(2)
                        self.zLastAbsolute = zLastAbsolute
                        zLastAbsolute = zAbsolute[z + 1]
                    }
                }
                z++

            return [(self.xAbsolute = xAbsolute), (self.yAbsolute = yAbsolute), (self.zAbsolute = zAbsolute)]
        }
    }

    $('#myTab a').click(function (e) {
        e.preventDefault();
        $('#myTab a[href="#setup"]').tab('show');
        $('#myTab a[href="#profiles"]').tab('show');
        $('#myTab a[href="#pid"]').tab('show');
        $('#myTab a[href="#layer"]').tab('show');
        $('#myTab a[href="#baseline"]').tab('show');
        $('#myTab a[href="#extruder"]').tab('show');
        $('#myTab a[href="#flow"]').tab('show');
        $('#myTab a[href="#current"]').tab('show');
        $('#myTab a[href="#retraction"]').tab('show');
        $('#myTab a[href="#temperature"]').tab('show');
        $('#myTab a[href="#acceleration"]').tab('show');
        $('#myTab a[href="#linear"]').tab('show');
    });

    $(document).ready(function(){
        $('[data-toggle="tooltip"]').tooltip();
    });

    $('.nav-tabs').button()

    OCTOPRINT_VIEWMODELS.push({
        construct: TtprintercalibrationViewModel,
        dependencies: [  "settingsViewModel"  ],
        elements: [ "#settings_plugin_TTPrinterCalibration", "#setup", "#modalFooter" ]
    });
});